//! Cursor implementation
use crate::{AccountId, BaseTx, Filter, RevId, Revision, Tag, TxId, Type};

#[derive(Debug, Clone, PartialEq, Eq, Hash)]
/// The primary filter is used to filter the transactions before applying the other filters. Think
/// of it like which key is used to filter transactions quickly before applying the other filters.
pub enum PrimaryFilter {
    /// By transaction ID
    TxId(Vec<TxId>),
    /// By revision ID
    Revision(Vec<RevId>),
    /// By accounts
    Account(Vec<AccountId>),
    /// By transaction type
    Type(Vec<Type>),
    /// By tags
    Tags(Vec<Tag>),
    /// By transaction status
    Stream,
}

#[derive(Debug, PartialEq, Eq, Hash, Clone)]
/// The cursor
pub struct Cursor {
    /// The primary filter
    pub primary_filter: PrimaryFilter,
    /// The secondary filter
    pub filter: Filter,
}

impl From<Filter> for Cursor {
    fn from(mut filter: Filter) -> Self {
        let primary_filter = if !filter.revisions.is_empty() {
            PrimaryFilter::Revision(std::mem::take(&mut filter.revisions))
        } else if !filter.ids.is_empty() {
            PrimaryFilter::TxId(std::mem::take(&mut filter.ids))
        } else if !filter.accounts.is_empty() {
            PrimaryFilter::Account(std::mem::take(&mut filter.accounts))
        } else if !filter.typ.is_empty() {
            PrimaryFilter::Type(std::mem::take(&mut filter.typ))
        } else if !filter.tags.is_empty() {
            PrimaryFilter::Tags(std::mem::take(&mut filter.tags))
        } else {
            PrimaryFilter::Stream
        };

        Self {
            primary_filter,
            filter: filter.prepare(),
        }
    }
}

impl Cursor {
    /// Check if the base transaction and the revision matches the current cursor filter
    pub fn matches(&self, base_tx: &BaseTx, revision: &Revision) -> bool {
        self.filter.matches(base_tx, revision)
    }
}
